const debug = require('debug')('bluefyre:config')

const fs = require('fs')
const user_config_path = `${process.cwd()}/bluefyre.json`

function deepClone(src, dst) {
  const keys = Object.keys(src)

  keys.forEach(key => {
    const val = src[key]

    if (val && !Array.isArray(val) && typeof val == 'object' && val.constructor && val.constructor.name == 'Object') {
      if (dst[key] === undefined) {
        dst[key] = {}
      }

      deepClone(val, dst[key])
    } else if (val !== undefined) {
      dst[key] = val
    }
  })
}

let user_config

try {
  user_config = require(user_config_path)
} catch (error) {
  debug('Failed to load config: %o', error)
  user_config = {}
}

let drop_fields_list = []

const drop_fields_env = process.env.BLUEFYRE_DROP_FIELDS

if (drop_fields_env && drop_fields_env.length) drop_fields_list = drop_fields_env.split(',')

const DEFAULT_CONFIG = {
  agent_id: (process.env.BLUEFYRE_AGENT_ID) ? process.env.BLUEFYRE_AGENT_ID:'AGENT_ID_NOT_SET',
  license_key: 'LICENSE_KEY_NOT_SET',
  collector_url: process.env.BLUEFYRE_API_COLLECTOR_URL || 'https://appsec-sensors.threatstack.com',
  collector_timeout_ms: 30000,
  event_queue_threshold: 1000,
  request_blocking: {
    sqli: process.env.BLUEFYRE_BLOCK_SQLI || false,
    nosqli: process.env.BLUEFYRE_BLOCK_NOSQLI || false,
    xss: process.env.BLUEFYRE_BLOCK_XSS || false
  },
  drop_fields: drop_fields_list  
}

const combined_config = {}
deepClone(DEFAULT_CONFIG, combined_config)
deepClone(user_config, combined_config)

// sigh, ok, the config needs to be properly refactored
// so that env vars always take precedence over file values
// without having to special-case stuff like this

if (drop_fields_list.length) {
  combined_config.drop_fields = drop_fields_list
} else if (!Array.isArray(combined_config.drop_fields)) {
  combined_config.drop_fields = []
}

let AGENT_VERSION

try {
  AGENT_VERSION = require('../package.json').version
} catch (error) {
  debug('Could not determine agent version.')
  AGENT_VERSION = 'UNKNOWN'
}

combined_config.version = AGENT_VERSION
debug("Bluefyre agent configuration: %o", combined_config)
module.exports = combined_config
