class BluefyreError extends Error {
  constructor(msg = 'Generic error', code = 'BFE_DEFAULT', details = {}) {
    super(msg)
    this.name = this.constructor.name
    this.code = code
    this.details = details
  }

  toString() {
    return `Bluefyre Error: ${this.message} (${this.code}).`
  }

  toJSON() {
    return {
      message: this.message,
      code: this.code,
      details: this.details,
      name: this.name
    }
  }

  get code() {
    return this._code
  }

  set code(code) {
    this._code = code
    return this
  }

  get details() {
    return this._details
  }

  set details(details) {
    this._details = details
    return this
  }
}

class RequestBlockedError extends BluefyreError {
  constructor(reason = 'Violation detected.', code = 'BFE_ILLEGAL_REQUEST', details) {
    super('Request blocked.', code, details)
    this.name = this.constructor.name
    this.reason = reason
    this.code = code
    this.details.isBlocked = true
  }

  get reason() {
    return this._reason
  }

  set reason(reason) {
    this._reason = reason
    return this
  }

  blocked(wasRequestBlocked=true) {
    this.details.isBlocked = wasRequestBlocked
  }
}

class BluefyreStackError {
  constructor(funcName) {
    const errObj = new Error()

    let interim = Error.prepareStackTrace

    Error.prepareStackTrace = function (err, stackTrace) {
      return stackTrace
    }

    // TODO: Change this
    // Adding the following disables stack trace collection. We're using this to test if this
    // improves agent performance
    // for more info = https://v8.dev/docs/stack-trace-api#basic-stack-traces
    //Error.stackTraceLimit = 0

    Error.captureStackTrace(errObj, funcName)
    this.stack = errObj.stack
    Error.prepareStackTrace = interim
  }
}

function handle_error(err, req, res) {
  if (err instanceof RequestBlockedError) {
    // collect relevant env/req info
    // send attack event
    __BLUEFYRE.reportIncident({ err, req })

    if (req.next) {
      return req.next(err)
    }

    res.statusCode = 403
    res.end('Request blocked')

    return
  }

  throw err
}

module.exports = {
  RequestBlockedError, BluefyreError, BluefyreStackError, handle_error
}
