// express-body-parser.js
// author: sabin.thomas

'use strict'
const utils = require('../../util/utils.js')
const debug = require('debug')('bluefyre:3rdparty:bodyParser')
const util = require('util')

module.exports = function initialize(bluefyreAgent, bodyParserModule) {
  var funcArr = utils.getPublicFuncs(bodyParserModule)

  utils.genericWrapConstructor(bluefyreAgent, bodyParserModule, 'express_body_parser')

  utils.genericWrap(bluefyreAgent, bodyParserModule, funcArr, 'express_body_parser'
  , bodyParserModule && bodyParserModule.prototype)

  for (let fn of funcArr) {
    utils.wrapMethod(bluefyreAgent, bodyParserModule, fn, 'body-parser', function (original) {
      return function () {
        return function __BF_parser_wrapper(options) {
          return bodyParserHook(bluefyreAgent, original.apply(null, arguments))
        }
      }
    })
  }
}

function bodyParserHook(agent, parser) {
  return function (req, res, next) {
    return parser(req, res, function (err) {
      if (err) return next(err)

      if (!req.__BF_body_parsed && req._body) {
        req.__BF_body_parsed = true

        debug('Body parsed: %o', req.body)

        agent.runHook('http_request', 'body-parser', {
          req
        })
      }

      return next()
    })
  }
}