// express-session.js
// author: sabin.thomas

'use strict'
const utils = require('../../../util/utils.js')
const util = require('util')
const debug = require('debug')('bluefyre:3rdparty:express:express-session')
const Event = require('../../../models/Event')
const clone = require('clone')
const BluefyreStackError = require('../../../errors').BluefyreStackError

class ExpressSessionEvent extends Event {
  constructor(method, payload = {}, trace) {
    super(payload, trace)
    this._data.module_name = 'express-session'
    this._data.method_name = method
    this._data.event_type = 'instrumentation'
  }
}

function wrapSession(bluefyreAgent, expressSessModule) {
  if (!expressSessModule){
    debug('No express-session module');
    return
  }
  if (!bluefyreAgent){
    debug('No Bluefyre Agent');
    return
  }

  const orig_expressSession = expressSessModule

  var session_module = function session_wrap(user_options) {
    const _original_opts = clone(user_options)
    const session = orig_expressSession.apply(orig_expressSession, arguments)
    const _effective_opts = clone(user_options) // session module modifies these
    const ev = new ExpressSessionEvent('constructor', { config: { user: _original_opts, effective: _effective_opts } }, new BluefyreStackError(session_wrap))

    bluefyreAgent.setEvent(ev)

    debug('Instrumented express-session/session')

    return session
  }

  Object.keys(expressSessModule).forEach(key => {
    session_module[key] = orig_expressSession[key]
  })

  debug('Instrumented express-session/session.')

  return session_module
}

module.exports = function initialize(bluefyreAgent, expressSessModule) {
    debug('Loading express-session wrapper...')

    if (!expressSessModule || !bluefyreAgent) {
      return debug('Did not load express-session-wrapped cannot load.')
    }

    return wrapSession(bluefyreAgent, expressSessModule)
}
