// express-session-cookie.js
// author: sabin.thomas

'use strict'
var utils = require('../../../../util/utils')
const util = require('util')
const debug = require('debug')('bluefyre:3rdparty:express:express-session:cookie')
const Event = require('../../../../models/Event')
const clone = require('clone')
const BluefyreStackError = require('../../../../errors').BluefyreStackError

class ExpressSessionCookieEvent extends Event {
  constructor(method, payload = {}, trace) {
    super(payload, trace)
    this._data.module_name = 'express-session/cookie'
    this._data.method_name = method
    this._data.event_type = 'instrumentation'
  }
}

function wrapCookie(bluefyreAgent, expressSessCookieModule){
  if (!expressSessCookieModule){
    debug('No express-session-cookie module');
    return
  }

  if (!bluefyreAgent){
    debug('No Bluefyre Agent');
    return
  }

  const cookie_wrapper = function cookie_wrapper_fn(options) {
    const instance = new expressSessCookieModule(options)
    bluefyreAgent.setEvent(new ExpressSessionCookieEvent('cookie', { config: options }, new BluefyreStackError(cookie_wrapper_fn)))
    return instance
  }

  debug('express-session/cookie instrumented.')

  return cookie_wrapper
}


module.exports = function initialize(bluefyreAgent, expressSessCookieModule) {
    debug('Loading Express-session-Cookie wrapper');

    if (!expressSessCookieModule){
      debug('Did not load express-session-cookie successfully')
    }

    return wrapCookie(bluefyreAgent, expressSessCookieModule)
}
