const util = require('util')
const Event = require('../../../models/Event')
const BluefyreStackError = require('../../../errors').BluefyreStackError

class HapiEvent extends Event {
  constructor(method, payload = {}, trace) {
    super(payload, trace)
    this._data.module_name = 'hapi'
    this._data.method_name = method
    this._data.event_type = 'instrumentation'
  }
}

module.exports = (agent, hapi) => {
  const Server = hapi.Server

  hapi.Server = function hapiServerWrap(options) {
    const server = new Server(options)

    agent.setEvent(new HapiEvent('new_server', {config: options}, new BluefyreStackError(hapiServerWrap)))

    const orig_connection = server.connection
    server.connection = function hapiConnWrap() {
      const connection = orig_connection.apply(server, arguments)

      agent.setEvent(new HapiEvent('connection', { config: arguments[0] }, new BluefyreStackError(hapiConnWrap)))

      return connection
    }

    const orig_route = server.route

    server.route = function hapiRouteWrap() {
      agent.setEvent(new HapiEvent('route', {config: arguments[0]}, new BluefyreStackError(hapiRouteWrap)))

      const orig_handler = arguments[0].handler
      arguments[0].handler = function hapiHandlerWrap(request, reply) {
        agent.setEvent(new HapiEvent('request', {headers: request.headers}, new BluefyreStackError(hapiHandlerWrap)))

        return orig_handler.apply(orig_handler, arguments)
      }

      return orig_route.apply(server, arguments)
    }

    return server
  }

  return hapi
}