// mongodb.js
// author: sabin.thomas

'use strict'
const utils = require('../../util/utils.js')
const util = require('util')
const debug = require('debug')('bluefyre:3rdparty:mongodb')
const namespace = require('../../util/namespace')

module.exports = function initialize(bluefyreAgent, mongodbModule) {
    debug('Loading mongodb wrapper');
    if (!mongodbModule){
      debug('Did not load mongodb successfully')
    }
    var arrFunc = ['save','find','deleteOne','deleteMany',
      'insertOne','insertMany','bulkWrite','insert','updateOne','replaceOne'
      ,'updateMany','update','removeOne','deleteMany','removeMany','remove'
      ,'findOne','rename','drop','createIndex','createIndexes','dropIndex','dropIndexes','dropAllIndexes'
      ,'reIndex','listIndexes','ensureIndex','indexExists','indexInformation','count','distinct'
      ,'indexes','stats','findOneAndDelete','findOneAndReplace','findOneAndUpdate'
      ,'findAndModify','findAndRemove','aggregate','parallelCollectionScan'
      ,'geoNear','geoHaystackSearch','group','mapReduce','initializeUnorderedBulkOp','initializeOrderedBulkOp']

    utils.genericWrap(bluefyreAgent, mongodbModule, arrFunc, 'mongodb_collection'
      , mongodbModule && mongodbModule['Collection'] && mongodbModule['Collection'].prototype)

  const cursor_methods = utils.getPublicFuncs(mongodbModule.Cursor.prototype)

  cursor_methods.forEach(fn => {
    const original = mongodbModule.Cursor.prototype[fn]

    mongodbModule.Cursor.prototype[fn] = function () {
      const args = utils.argsToArray.apply(this, arguments)

      if (typeof args[args.length - 1] == 'function') {
        const cb = args[args.length - 1]
        args[args.length - 1] = namespace.bind(cb)
      }

      return original.apply(this, args)
    }
  })

  const Client = mongodbModule.MongoClient

  utils.genericWrap(bluefyreAgent, Client, ['connect'], 'mongodb.MongoClient', Client)
}
