// http.js
// author: sabin.thomas
// http Nodejs Library wrappers

'use strict'
const http = require('http');
const domain = require('domain')
const utils = require('../../util/utils.js')
const namespace = require('../../util/namespace')
const uuid = require('uuid')
const debug = require('debug')('bluefyre:core:http')
const errors = require('../../errors')

module.exports = function initialize(bluefyreAgent) {
  utils.genericWrap(bluefyreAgent, http, ['createServer'], 'http', http)
  utils.genericWrap(bluefyreAgent, http, ['writeHead'], 'http'
    , http && http['ServerResponse'] && http['ServerResponse'].prototype)

  const emit = http.Server.prototype.emit

  http.Server.prototype.emit = function __BF_http_Server_emit(ev, req, res) {
    if (ev == 'request') {
      onRequest.call(this, emit, req, res, bluefyreAgent)
    } else {
      emit.apply(this, arguments)
    }
  }
}

function onRequest(emit, req, res, agent) {
  const self = this

  var d = domain.create()

  d.add(req)
  d.add(res)

  d.on('error', err => {
    if (namespace.active) {
      errors.handle_error(err, req, res)
    } else {
      namespace.run(function () {
        req &&
          req.__bluefyre &&
          req.__bluefyre.req &&
          namespace.set('req_id', req.__bluefyre.req.id)
        errors.handle_error(err, req, res)
      })
    }
  })

  d.run(function() {
    namespace.run(function () {
      namespace.set('domain', d)
      requestInDomain.call(self, emit, req, res, agent)
    })
  })
}

function requestInDomain(emit, req, res, agent) {
  agent.parseReqInfo(req)

  namespace.bindEmitter(req)
  namespace.bindEmitter(res)

  namespace.set('req', req)
  namespace.set('req_id', req.__bluefyre.req.id)
  namespace.set('res', res)

  req.__BF_NAMESPACE = namespace

  debug('HTTP request wrapped. Req ID: %s.', req.__bluefyre.req.id)

  emit.call(this, 'request', req, res)
}

