// module.js
// author: sabin.thomas

'use strict'
var utils = require('../../util/utils.js')
var path = require('path')
const util = require('util')
const fs = require('fs')
const debug = require('debug')('bluefyre:core:module')
const Event = require('../../models/Event')
const FRAMEWORKLIBS = require('./instrumented_modules').FRAMEWORKLIBS
const Module = require('module')

class ModuleEvent extends Event {
  constructor(method, payload = {}) {
    super(payload)
    this._data.module_name = 'module'
    this._data.method_name = method
    this._data.event_type = 'instrumentation'
  }
}


function wrapLoad(bluefyreAgent, module_original) {
  if (!module_original){
    debug('No module')
    return
  }

  if (!bluefyreAgent){
    debug('No Bluefyre Agent')
    return
  }

  const loadOrigFunc = module_original._load

  module_original._load = function wrappedLoad(modulePath, parent) {
    const ev = new ModuleEvent('load', { path: modulePath })
    bluefyreAgent.setEvent(ev)

    const loaded_module = loadOrigFunc.apply(this, arguments)

    return wrapAfterLoad(loaded_module, modulePath, parent, bluefyreAgent)
  }

  module_original._load.__BF_unwrap = function () {
    module_original._load = loadOrigFunc
    delete module_original._load.__BF_unwrap
  }
}

function wrapAfterLoad(moduleInstance, modulePath, parent, bluefyreAgent) {
  var moduleName = path.basename(modulePath)

  if (modulePath in FRAMEWORKLIBS) {
    debug(moduleName, 'found in instrumentation', modulePath)

    try {
      let module_info

      if (FRAMEWORKLIBS[moduleName]) {
        debug('FRAMEWORKLIBS[moduleName]', FRAMEWORKLIBS[moduleName])
        module_info = FRAMEWORKLIBS[moduleName]
      } else if (FRAMEWORKLIBS[modulePath]) {
        debug('FRAMEWORKLIBS[modulePath]', FRAMEWORKLIBS[modulePath])
        module_info = FRAMEWORKLIBS[modulePath]
      } else {
        debug('Cant find', moduleName, 'hook')
        return moduleInstance
      }

      const cachedModule = getCachedModule(modulePath, parent)
      const instrumented = require(module_info)(bluefyreAgent, moduleInstance, cachedModule)

      if (instrumented) {
        debug(`Returning module wrapper for %s`, moduleName || modulePath)
        return instrumented
      }
    } catch (err) {
      // send error event
      debug('Errors trying to load hook for', moduleName, err)
    }
  }

  return moduleInstance
}

function getCachedModule(name, parent) {
  return Module._cache[Module._resolveFilename(name, parent)]
}


module.exports = function initialize(bluefyreAgent, module_original) {
  debug('Loading Module wrapper')

  if (!module_original){
    debug('Did not load module successfully')
  }

  wrapLoad(bluefyreAgent, module_original)
}
