// math.js
// author: sabin.thomas

'use strict'
var utils = require('../../util/utils.js')
const Event = require('../../models/Event')
const BluefyreStackError = require('../../errors').BluefyreStackError
const debug = require('debug')('bluefyre:js:math')

class MathEvent extends Event {
  constructor(method, payload = {}, trace) {
    super(payload, trace)
    this._data.module_name = 'math' // need to distinguish native JS classes from native node modules and from npm modules3
    this._data.method_name = method
    this._data.event_type = 'instrumentation'
  }
}

function wrapRandom(bluefyreAgent) {
  if (!bluefyreAgent){
    debug('No Bluefyre Agent');
  }

  var origFunc = Math.random;

  Math.random = function wrappedRandom(){
    if (typeof origFunc != 'function'){
      debug('origFunc is not provided');
      return
    }

    bluefyreAgent.setEvent(new MathEvent('random', {}, new BluefyreStackError(wrappedRandom)));
    return origFunc.apply(this, arguments);
  }
}

// TODO: make this function a generic wrapper, there is repeated functionality between wrapRandom and wrapMax
function wrapMax(bluefyreAgent){
  if (!bluefyreAgent){
    debug('No Bluefyre Agent');
  }

  var origFunc = Math.max;
  Math.max = function wrappedMax(){
    if (typeof origFunc != 'function'){
      debug('origFunc is not provided')
      return
    }

    var args = utils.argsToArray.apply(this, arguments)
    bluefyreAgent.setEvent(new MathEvent('max', { arguments: args }, new BluefyreStackError(wrappedMax)));
    return origFunc.apply(this,arguments);
  }
}

module.exports = function initialize(bluefyreAgent) {
    wrapRandom(bluefyreAgent);
    wrapMax(bluefyreAgent);
}
