const uuid = require('uuid')
const clone = require('clone')
const namespace = require('../util/namespace')
const debug = require('debug')('bluefyre:models:event')

class Event {
  constructor(payload = {}, trace) {
    this._data = {}

    try {
      this._data.payload = clone(payload)
    } catch(e) {
      if(e instanceof TypeError || e instanceof ReferenceError){
        debug('Event, trying to clone a payload without a setter defined')
      }
    }

    if (trace && trace.stack && trace.stack[0]) {
      this._data.file_path = trace.stack[0].getFileName()
      this._data.line_num = trace.stack[0].getLineNumber()
    }

    this._captureMetadata()
  }

  _captureMetadata() {
    this._data.timestamp = new Date().toISOString()
    this._data.event_id = uuid.v4()
    this._data.request_id = namespace.get('req_id')
    this._data.event_sourceip = namespace.get('req') &&
      namespace.get('req').__bluefyre &&
      namespace.get('req').__bluefyre.req &&
      namespace.get('req').__bluefyre.req.ip_address

    //this._data.event_sourceip = 'something' + new Date()
}

  payload(p) {
    this._data.payload = clone(p)
  }

  toJSON() {
    var retObj
    try{
      retObj = clone(this._data)
    } catch(e){
      if(e instanceof TypeError || e instanceof ReferenceError){
        debug('Event, trying to clone a payload without a setter defined')
      }
    }
    return retObj
  }

  toString() {
    return JSON.stringify(this.toJSON())
  }
}

module.exports = Event
