/**
 * parse local packages
 */

const debug = require('debug')('bluefyre:util:package')
const Path = require('path')
const fs = require('fs')
const FILE_PKG_JSON = 'package.json'
const FILE_PKG_LOCK_JSON = 'package-lock.json'
const FILE_SHRINKWRAP = 'npm-shrinkwrap.json'

function getPkgJsonLocation(cb) {

  // two means of getting the path
  let appDirPath = Path.join(__dirname, '../../..')
  let appPath = Path.join(appDirPath, FILE_PKG_JSON)
  let pkgJsonLocation = process.cwd()

  if (fs.existsSync(appPath)) {
    pkgJsonLocation = appDirPath
  }
  debug('pkgJsonLocation', pkgJsonLocation)
  cb(pkgJsonLocation)
  
}

function getPackages(cb) {
  
  getPkgJsonLocation((pkgJsonLocation) => {
    let retObj = {}

    // let's read in the package.json
    let pkgJsonContentsFS = fs.readFileSync(Path.join(pkgJsonLocation, FILE_PKG_JSON));
    let pkgJSONcontents = JSON.parse(pkgJsonContentsFS)
    retObj.dependencies = (pkgJSONcontents && pkgJSONcontents.dependencies) ? pkgJSONcontents.dependencies : {}
    retObj.name = (pkgJSONcontents && pkgJSONcontents.name) ? pkgJSONcontents.name : {}
    retObj.version = (pkgJSONcontents && pkgJSONcontents.version) ? pkgJSONcontents.version : {}
    retObj.description = (pkgJSONcontents && pkgJSONcontents.description) ? pkgJSONcontents.description : {}
    retObj.graph = {}

    // let's read in the package-lock.json
    let pkgLockJsonContents = fs.readFileSync(Path.join(pkgJsonLocation, FILE_PKG_LOCK_JSON));
    pkgLockJsonContents = JSON.parse(pkgLockJsonContents)
    let dependencyGraph = (pkgLockJsonContents && pkgLockJsonContents.dependencies) ? pkgLockJsonContents.dependencies: {}
    Object.keys(dependencyGraph).map(function map_pkglock(item){
      let pkgObj = dependencyGraph[item]

      function trimObj(obj){
        if (obj['resolved']){
          delete obj['resolved']
        }
        if (obj['integrity']){
          delete obj['integrity']
        }
        if (obj['dependencies']){
          Object.keys(obj['dependencies']).map(item2 => {
            obj['dependencies'][item2] = trimObj(obj['dependencies'][item2])
          })
        }
        return obj
      }

      pkgObj = trimObj(pkgObj)
      retObj.graph[item] = pkgObj
    })
    
    //debug('retObj', retObj)    
    cb(retObj)
  })

}

module.exports = {
  getPackages: getPackages
}
