#include <nan.h>
#include <vector>
#include <string>
#include <stdio.h>
#include <strings.h>
#include <errno.h>
#include "libinjection.h"
#include "libinjection_sqli.h"
#include "libinjection_xss.h"

using namespace v8;
using namespace Nan;


NAN_METHOD(is_sqli) {
  int issqli = 0;

  if (info[0]->IsString()) {
    struct libinjection_sqli_state state;

    Nan::MaybeLocal<String> tmp = Nan::To<String>(info[0]);
    Local<String> local_string = tmp.ToLocalChecked();

    Nan::Utf8String val(local_string);
    std::string str(*val);

    const char *input = str.c_str();
    size_t slen = strlen(input);

    /* in real-world, you would url-decode the input, etc */

    libinjection_sqli_init(&state, input, slen, FLAG_NONE);
    issqli = libinjection_is_sqli(&state);

    if (issqli) {
      fprintf(stderr, "sqli detected with fingerprint of '%s'\n", state.fingerprint);
      info.GetReturnValue().Set(New<String>(state.fingerprint).ToLocalChecked());
      return;
    }
  } else {
    fprintf(stderr, "libinjection.is_sqli: input was not a string, ignoring.\n");
  }

  info.GetReturnValue().Set(Nan::Null());
}

NAN_METHOD(is_xss) {
  int isxss = 0;

  if (info[0]->IsString()) {

    Nan::MaybeLocal<String> tmp = Nan::To<String>(info[0]);
    Local<String> local_string = tmp.ToLocalChecked();

    Nan::Utf8String val(local_string);
    std::string str(*val);

    const char *input = str.c_str();
    size_t slen = strlen(input);

    /* in real-world, you would url-decode the input, etc */
    isxss = libinjection_is_xss(input, slen, FLAG_NONE);

    if (isxss) {
      fprintf(stderr, "xss detected - '%s'\n", input);
      info.GetReturnValue().Set(New<String>(input).ToLocalChecked());
      return;
    }
  } else {
    fprintf(stderr, "libinjection.is_xss: input was not a string, ignoring.\n");
  }

  info.GetReturnValue().Set(Nan::Null());
}

NAN_MODULE_INIT(Init) {
  Nan::Set(
      target,
      New<String>("is_sqli").ToLocalChecked(),
      GetFunction(New<FunctionTemplate>(is_sqli)).ToLocalChecked());
  Nan::Set(
      target,
      New<String>("is_xss").ToLocalChecked(),
      GetFunction(New<FunctionTemplate>(is_xss)).ToLocalChecked());      
}

NODE_MODULE(bluefyre, Init);
